/**********************************************************************\
**
**                         Copyright (c) 2009-2018 by
**                          HENSGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a licence and may be used and
**      copied only in accordance with the terms  of  such  licence  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      Exo_Demo_FormFeed.c
**  Facility:    Hengstler Extendo Linux Printer Driver Package / S684018
**  Application: Extendo API test application for Form Feed under linux
**  Version:     R3-V1_04e-BETA-2018AUG14
**  Environment: Linux Debian/Ubuntu/SUSE
**  Compiler:    gcc
**  Authors:     L. Tkac
**  Abstract:    Main file of the Extendo API Demo application for Form feed process
**  History:     R3-V1.04a / 2018MAR23 / TKA / Initial version
**               R3-V1.04e / 2018AUG14 / TKA / Return value of printer handle change from 'int' to 'intptr_t' 
**                                             due to 32/64bit incompatibilities on some 64bit distributions (Ubuntu 18.04-64bit)

**
\**********************************************************************/

/*
** INCLUDES
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "Exo_Api.h"

   int main (int argc, char *argv[])
   {  
      intptr_t          iPrinter          =0;
      int               iWritenData       =0;      
      unsigned char     cmd_FormFeed[]    = {0x0C};                                       // Form feed
      unsigned char     cmd_Cut[]         = {0x1B, 0xF0, 0x06, 0x01, 0x02};               // Cut cmd_Cut
      char              *printer_name     = NULL;			
      
      //-- Parameter count must be 2, first is application name , second is printer path
      if (argc < 2){
        //-- Debug message
         printf("\nERROR: No printer path available or page length missing, example usage : %s \"/dev/bus/usb/001/011\"\n\n", argv[0]);
        
        //-- Exit application
        goto EXIT;
      } 

      //-- Debug message         
      printf ("\n-------------------------------------------------\n");      
      printf ("--- START OF APPLICATION (%s)\n",argv[0]);
      printf ("-------------------------------------------------\n");      
      
      
      //-- Allocate and Copy printer name
      printer_name = (char *) malloc (sizeof(char) * (strlen(argv[1]) +1 ));
      if(printer_name!= NULL){
         
         //-- Copy printer name    
         memcpy(printer_name, argv[1], strlen(argv[1]) );
         printer_name[strlen(argv[1])] = '\0';
         
      }else {
         
         //-- Debug message         
         printf ("--- ERROR: COULD NOT ALLOCATE MEMORY !\n");
         
         //-- Stop application
         goto EXIT; 
      }
      
      //-- Open printer connection
      iPrinter = exo_api_printer_open(&printer_name[0]);

      //-- Debug message      
      printf ("\n-------------------------------------------------\n");      
      printf ("--- Printer name \t= %s\n--- Printer handler \t= %lu\n",argv[1],iPrinter);      
      printf ("-------------------------------------------------\n");      
      
      //-- Check if printer was opened  and set page length 
      if ( iPrinter >=0 )	
      {            
         //-- Clear pending data
         exo_api_printer_clear (iPrinter);
         
         //-- Write cmd to printer
         if ((iWritenData = exo_api_printer_write(iPrinter, cmd_FormFeed, sizeof(cmd_FormFeed), 1000)) != sizeof(cmd_FormFeed)){
            //-- Debug message         
            printf ("--- ERROR: COULD NOT SENT FORM FEED CMD !\n");                                       
            
            //-- Stop application
            goto EXIT;
         } else 
            //-- Debug message         
            printf ("--- FORM FEED CMD SENT OK\n");                                       
         
         
         //-- Write cmd to printer
         if ((iWritenData = exo_api_printer_write(iPrinter, cmd_Cut, sizeof(cmd_Cut), 1000)) != sizeof(cmd_Cut)){
            //-- Debug message         
            printf ("--- ERROR: COULD NOT SENT FULL CUT CMD !\n");                                       
         
            //-- Stop application
            goto EXIT;
            
         } else 
            //-- Debug message         
            printf ("--- FULL CUT CMD SENT OK\n");                                       
         
         //-- Debug message                  
         printf ("-------------------------------------------------\n");      
         
         //-- Close printer
         exo_api_printer_close(iPrinter);
         iPrinter =0;
         
      }else {
         //-- Debug message         
         printf ("--- ERROR: OPENING PRINTER = %s \n",argv[1]);                                       
         
         //-- Stop application
         goto EXIT;         
      }      
      
      //-- Debug message  
      printf ("\n-------------------------------------------------\n");            
      printf ("--- END OF APPLICATION (%s)\n",argv[0]);
      printf ("-------------------------------------------------\n");      
      
EXIT :
      
      //-- Free memory
      if(printer_name!= NULL){
         free(printer_name);
      }
      printer_name = NULL;

      //-- Close printer
      if(iPrinter != 0){
         exo_api_printer_close(iPrinter);      
      }
      
      return 0;
   } 
