/**********************************************************************\
**
**                         Copyright (c) 2009-2018 by
**                          HENSGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a licence and may be used and
**      copied only in accordance with the terms  of  such  licence  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      Exo_Demo_InquiryStatus.c
**  Facility:    Hengstler Extendo Linux Printer Driver Package / S684018
**  Application: Extendo API test application for Inquiry status under linux
**  Version:     R3-V1_04e-BETA-2018AUG14
**  Environment: Linux Debian/Ubuntu/SUSE
**  Compiler:    gcc
**  Authors:     L. Tkac
**  Abstract:    Main file of the Extendo API Demo application for Inquiry printer status
**  History:     R3-V1.04a / 2018MAR23 / TKA / Initial version
**               R3-V1.04b / 2018JUN25 / TKA / Changed inquiring into 2 packages(header+body)
**                                             Info message structure updated
**               R3-V1.04e / 2018AUG14 / TKA / Return value of printer handle change from 'int' to 'intptr_t' 
**                                             due to 32/64bit incompatibilities on some 64bit distributions (Ubuntu 18.04-64bit)
**
\**********************************************************************/

/*
** INCLUDES
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "Exo_Api.h"

   int main (int argc, char *argv[])
   {  
      char              *printer_name                    = NULL;
      intptr_t          iPrinter                         =0;
      int               iWritenData                      =0;      
      int               iReadData                        =0;      
      unsigned char     cmd_PrinterStatusRequest[]       = {0x1D, 0x61, 0x02};                                       // Inquiry printer info status , for print status last byte 0x01
      unsigned char     cmd_PrinterStatusHeader[sizeof(TS_EXO_API_PCKT_HEADER) ];                           
      unsigned char     cmd_PrinterStatusData[sizeof(TS_EXO_API_PCKT_PRINTER_INFO)];                                 // Inquiring printer info package body, for printer status package change to TS_EXO_API_PCKT_PRINTER_STATUS
      
      //-- Parameter count must be 2, first is application name , second is printer path
      if (argc < 2){
        //-- Debug message
         printf("\nERROR: No printer path available or page length missing, example usage : %s \"/dev/bus/usb/001/011\"\n\n", argv[0]);
        
        //-- Exit application
        goto EXIT;
      } 

      //-- Debug message         
      printf ("\n-------------------------------------------------\n");      
      printf ("--- START OF APPLICATION (%s)\n",argv[0]);
      printf ("-------------------------------------------------\n");      
      
      
      //-- Allocate and Copy printer name
      printer_name = (char *) malloc (sizeof(char) * (strlen(argv[1]) +1 ));
      if(printer_name!= NULL){
	
         //-- Copy printer name    
         memcpy(printer_name, argv[1], strlen(argv[1]) );
         printer_name[strlen(argv[1])] = '\0';
         
      }else {
         
         //-- Debug message         
         printf ("--- ERROR: COULD NOT ALLOCATE MEMORY !\n");
         
         //-- Stop application
         goto EXIT; 
      }
      
      //-- Open printer connection
      iPrinter = exo_api_printer_open(&printer_name[0]);

      //-- Debug message      
      printf ("\n-------------------------------------------------\n");      
      printf ("--- Printer name \t= %s\n--- Printer handler \t= %lu\n",argv[1],iPrinter);      
      printf ("-------------------------------------------------\n");      
      
      //-- Check if printer was opened  and set page length 
      if ( iPrinter >=0 )	
      {            
         //-- Clear pending data
         exo_api_printer_clear (iPrinter);
                          
         //-- Write cmd to printer
         if ((iWritenData = exo_api_printer_write(iPrinter, cmd_PrinterStatusRequest, sizeof(cmd_PrinterStatusRequest), 1000)) != sizeof(cmd_PrinterStatusRequest)){
               
            //-- Debug message         
            printf ("--- ERROR: COULD NOT SENT PRINTER STATUS INQUIRY CMD !\n");                                       
            
            //-- Stop application
            goto EXIT;
         } else {
            //-- Debug message         
            printf ("--- PRINTER STATUS INQUIRY CMD SENT: \t\t");                                       
            int iPos =0;
            for (iPos =0; iPos <sizeof(cmd_PrinterStatusRequest); iPos++)      {            
               fprintf(stdout, "[%2.2X] ", cmd_PrinterStatusRequest[iPos]); fflush(stdout);            
               if(   (iPos>0)
                  && ((iPos+1)%16)==0
               ){
                  //-- Debug message         
                  printf ("\n");                                                      
                  printf ("\t\t\t\t\t\t");                                                      
               }
            }
            //-- Debug message         
            printf ("\n");                                                      
         }
                    
          //-- Wait ... ms for sync to become effective
          //M_EXO_SLEEP_MS(1000);
                    
         //-- Read cmd to printer
         memset(cmd_PrinterStatusHeader, 0x00, sizeof(cmd_PrinterStatusHeader));
         if ((iReadData = exo_api_printer_read(iPrinter, cmd_PrinterStatusHeader, sizeof(cmd_PrinterStatusHeader), 1000)) != sizeof(cmd_PrinterStatusHeader)){

            //-- Debug message         
            printf ("--- ERROR: COULD NOT READ PRINTER INFO HEADER DATA !\n");                                       
            
            //-- Stop application
            goto EXIT;
            
         } else {
            
            //-- Debug message         
            printf ("--- PRINTER INFO STATUS READ HEADER DATA: \t");                                       
            int iPos =0;
            for (iPos =0; iPos <sizeof(cmd_PrinterStatusHeader); iPos++)      {            
               fprintf(stdout, "[%2.2X] ", cmd_PrinterStatusHeader[iPos]); fflush(stdout);            
               if(   (iPos>0)
                  && ((iPos+1)%16)==0
               ){
                  //-- Debug message         
                  printf ("\n");                                                      
                  printf ("\t\t\t\t\t\t");                                                      
               }
            }
            //-- Debug message         
            printf ("\n");                                                      
            
            
            //-- Read cmd to printer
            memset(cmd_PrinterStatusData, 0x00, sizeof(cmd_PrinterStatusData));
            if ((iReadData = exo_api_printer_read(iPrinter, cmd_PrinterStatusData, sizeof(cmd_PrinterStatusData), 1000)) != sizeof(cmd_PrinterStatusData)){
               
               //-- Debug message         
               printf ("--- ERROR: COULD NOT READ PRINTER INFO BODY DATA !\n");                                       
               
               //-- Stop application
               goto EXIT;
               
            } else {
               
            }
            
            //-- Debug message         
            printf ("--- PRINTER INFO STATUS READ BODY DATA: \t");                                                   
            for (iPos =0; iPos <sizeof(cmd_PrinterStatusData); iPos++)      {            
               fprintf(stdout, "[%2.2X] ", cmd_PrinterStatusData[iPos]); fflush(stdout);            
               if(   (iPos>0)
                  && ((iPos+1)%16)==0
               ){
                  //-- Debug message         
                  printf ("\n");                                                      
                  printf ("\t\t\t\t\t\t");                                                      
               }
               
            }
            //-- Debug message         
            printf ("\n");                                                      
            
         }         
        
         //-- Debug message                  
         printf ("-------------------------------------------------\n");      
         
         //-- Close printer
         exo_api_printer_close(iPrinter);
         iPrinter =0;
         
      }else {
         //-- Debug message         
         printf ("--- ERROR: OPENING PRINTER = %s \n",argv[1]);                                       
         
         //-- Stop application
         goto EXIT;         
      }      
      
      //-- Debug message  
      printf ("\n-------------------------------------------------\n");            
      printf ("--- END OF APPLICATION (%s)\n",argv[0]);
      printf ("-------------------------------------------------\n");      
      
EXIT :
      
      //-- Free memory
      if (printer_name!= NULL){
         free(printer_name);
      }
      printer_name = NULL;

      //-- Close printer
      if(iPrinter != 0){
         exo_api_printer_close(iPrinter);      
      }
      
      return 0;
   } 
