/**********************************************************************\
**
**                         Copyright (c) 2009-2018 by
**                          HENSGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a licence and may be used and
**      copied only in accordance with the terms  of  such  licence  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      Exo_Demo_PageLength.c
**  Facility:    Hengstler Extendo Linux Printer Driver Package / S684018
**  Application: Extendo API test application for page length setup under linux
**  Version:     R3-V1_04e-BETA-2018AUG14
**  Environment: Linux Debian/Ubuntu/SUSE
**  Compiler:    gcc
**  Authors:     L. Tkac
**  Abstract:    Main file of the Extendo API Demo application for setting Page Length 
**  History:     R3-V1.02d  / 2015MAY07 / TKA / Initial version
**               R3-V1.04a  / 2018MAR23 / TKA / Code clean up, edded comments,
**               Modified to only page length settings
**               R3-V1.04e / 2018AUG14 / TKA / Return value of printer handle change from 'int' to 'intptr_t' 
**                                             due to 32/64bit incompatibilities on some 64bit distributions (Ubuntu 18.04-64bit)

**
\**********************************************************************/

/*
** INCLUDES
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "Exo_Api.h"

   int main (int argc, char *argv[])
   {  
      intptr_t          iPrinter          =0;
      int               iWritenData       =0;      
      unsigned char     cmd_HardReset[]   = {0x1B, 0xF2, 0x03, 0x00};                     // Hard reset 
      unsigned char     cmd_FormLength[]  = {0x1B, 0xF1, 0x01, 0x03, 0x0A, 0x00, 0x00};   // Page size 
      char              *printer_name     = NULL;			
      int               page_size         = 300;
      
      //-- Parameter count must be 3, first is application name , second is printer path, third is page length 1/10 in mm
      if (argc < 3){
        //-- Debug message
         printf("\nERROR: No printer path available or page length missing, example usage : %s \"/dev/bus/usb/001/011\" 1255 \n\n", argv[0]);
        
        //-- Exit application
        goto EXIT;
      } 

      //-- Debug message         
      printf ("\n-------------------------------------------------\n");      
      printf ("--- START OF APPLICATION (%s)\n",argv[0]);
      printf ("-------------------------------------------------\n");      
      
      
      //-- Allocate and Copy printer name
      printer_name = (char *) malloc (sizeof(char) * (strlen(argv[1]) +1 ));
      if(printer_name!= NULL){
        
	//-- Copy printer name    
         memcpy(printer_name, argv[1], strlen(argv[1]) );
         printer_name[strlen(argv[1])] = '\0';

	
      }else {
         
         //-- Debug message         
         printf ("--- ERROR: COULD NOT ALLOCATE MEMORY !\n");
         
         //-- Stop application
         goto EXIT; 
      }
      
      //-- Get page length from parameter
      page_size = atoi(argv[2]);
      
      //-- Open printer connection
      iPrinter = exo_api_printer_open(&printer_name[0]);

      //-- Debug message      
      printf ("\n-------------------------------------------------\n");      
      printf ("--- Printer name \t= %s\n--- Page length \t= %.1lf mm\n--- Printer handler \t= %lu\n",argv[1],(float)((float)page_size/10),iPrinter);      
      printf ("-------------------------------------------------\n");      
      
      //-- Check if printer was opened  and set page length 
      if ( iPrinter >=0 )	
      {            
         //-- Clear pending data
         exo_api_printer_clear (iPrinter);

         //-- Set page lengt in mm, // 125.5mm = 1255 = n = [0x04, 0xE7]
         cmd_FormLength[5] = (unsigned char)(page_size/0xFF); 
         cmd_FormLength[6] = (unsigned char)(page_size%0xFF); 
         
         //-- Write cmd to printer
         if ((iWritenData = exo_api_printer_write(iPrinter, cmd_FormLength, sizeof(cmd_FormLength), 1000)) != sizeof(cmd_FormLength)){
            //-- Debug message         
            printf ("--- ERROR: COULD NOT SENT PAGE LENGTH CMD !\n");                                       
            
            //-- Stop application
            goto EXIT;
         } else 
            //-- Debug message         
            printf ("--- PAGE LENGTH CMD SENT OK\n");                                       
         
         
         //-- Write cmd to printer
         if ((iWritenData = exo_api_printer_write(iPrinter, cmd_HardReset, sizeof(cmd_HardReset), 1000)) != sizeof(cmd_HardReset)){
            //-- Debug message         
            printf ("--- ERROR: COULD NOT SENT HARD RESET CMD !\n");                                       
         
            //-- Stop application
            goto EXIT;
            
         } else 
            //-- Debug message         
            printf ("--- HARD RESET CMD SENT OK\n");                                       
         
         //-- Debug message                  
         printf ("-------------------------------------------------\n");      
         
         //-- Close printer
         exo_api_printer_close(iPrinter);
         iPrinter =0;
         
      }else {
         //-- Debug message         
         printf ("--- ERROR: OPENING PRINTER = %s \n",argv[1]);                                       
         
         //-- Stop application
         goto EXIT;         
      }      
      
      //-- Debug message  
      printf ("\n-------------------------------------------------\n");            
      printf ("--- END OF APPLICATION (%s)\n",argv[0]);
      printf ("-------------------------------------------------\n");      
      
EXIT :
      
      //-- Free memory
      if (printer_name!= NULL)
	free(printer_name);
      printer_name = NULL;

      //-- Close printer
      if(iPrinter != 0)
         exo_api_printer_close(iPrinter);      
      
      return 0;
   } 
